unit WorkUnit;

  //////////////////////////////////////////////////////
  //      MuzCat v1.43   //
  // (C) 2004 PA (ZLDO)                               //
  //////////////////////////////////////////////////////

interface

uses
  Windows, KOL, FormatHeader,
  AAC,        // *.aac
  Monkey,     // *.ape;*.mpc
  flac,       // *.flac
  MpegAudio,  // *.mp3;*.mp2
  MpegPlus,   // *.mpc;*.mp+
  TwinVQ,     // *.vqf
  WMA,        // *.wma;*.asf
  CDATrack    // *.cda
  ;


type
  TGlobalInfBuf = record
    Title:string;       // 
    Artist:string;      // 
    Album:string;       // 
    Comment:string;     // 
    Genre:string;       // 
    Date:string;        // 
    Time:integer;       // 
    BitRate:integer;    //  
    TrackNo:byte;       //  
    TagPr:boolean;      //    
  end;

const
  FeeldNameToFieldSeparator = ' - ';
  LineSeparator = #13#10;

  //       
  fn_Song_title = '';
  fn_Artist_name = '';
  fn_Album_title = '';
  fn_Track_number = '';
  fn_Release_year = '';
  fn_Genre_name = '';
  fn_Comment = '';
  fn_Composer = '';
  fn_Encoder = '';
  fn_Copyright = '';
  fn_Language = '';
  fn_URL_link = '';
  fn_Header_type_name = ' ';
  fn_MPEG_version_name = ' MPEG';
  fn_Profile_name = ' ';
  fn_Number_of_channels = ' ';
  fn_Sample_rate = ' ';
  fn_Bit_rate = ' ';
  fn_Encoder_version = ' ';
  fn_Compression_level = ' ';
  fn_Channel_mode = ' ';
  fn_Peak_level_ratio = '';
  fn_Compression_ratio = '  (%)';
  fn_MPEG_layer_name = ' MPEG';
  fn_Original_file_name = '  ';
  fn_Rating = '';
  fn_Lyrics = '';
  fn_Channels = '';

var
  GlobalInfBuf: TGlobalInfBuf; //       MuzCat
                               //           DLL

  FileInfoBuf: string; //    

  AppHandle: THandle;

  Description: string = ' ATLBased v1.0    Audio Tools Library (by Jurgen Faul).   KOL -  .'#13#10+
                        '     : ID3v1, ID3v2, WMA, TwinVQ, OggVorbis, APE, FLAC.'#13#10#13#10+
                        '() 2004 PA';
  dllExt: string;
  FormatList: PStrList = nil;


function ExtToIdx(Const Ext:string):integer;

//=====================    ===========================//
function Read_aacTag(Const Info: PBasicInfoRec):boolean; {idx = 0}
function Read_MpegPlusTag(Const Info: PBasicInfoRec):boolean; {idx = 6}
function Read_apeTag(Const Info: PBasicInfoRec):boolean; {idx = 1..2}
function Read_FLACTag(Const Info: PBasicInfoRec):boolean; {idx = 3}
function Read_MpegTag(Const Info: PBasicInfoRec):boolean; {idx = 4..5}
function Read_TwinVQTag(Const Info: PBasicInfoRec):boolean; {idx = 7}
function Read_wmaTag(Const Info: PBasicInfoRec):boolean; {idx = 8..9}
function Read_CDATag(Const Info: PBasicInfoRec):boolean; {idx = 10}
//====end==============    ===========================//

//=====================    ===========================//
function Write_aacTag(Const Info: PBasicInfoRec):boolean; {idx = 0}
function Write_MpegPlusTag(Const Info: PBasicInfoRec):boolean; {idx = 6}
function Write_apeTag(Const Info: PBasicInfoRec):boolean; {idx = 1..2}
function Write_FLACTag(Const Info: PBasicInfoRec):boolean; {idx = 3}
function Write_MpegTag(Const Info: PBasicInfoRec):boolean; {idx = 4..5}
function Write_TwinVQTag(Const Info: PBasicInfoRec):boolean; {idx = 7}
function Write_wmaTag(Const Info: PBasicInfoRec):boolean; {idx = 8..9}
//====end==============    ===========================//

procedure Info_WMA(Const FileName:PChar;Var info:PChar);
procedure Info_TwinVQ(Const FileName:PChar;Var info:PChar);
procedure Info_MpegAudio(Const FileName:PChar;Var info:PChar);
procedure Info_FLAC(Const FileName:PChar;Var info:PChar);
procedure Info_ape(Const FileName:PChar;Var info:PChar);
procedure Info_MpegPlus(Const FileName:PChar;Var info:PChar);
procedure Info_aac(Const FileName:PChar;Var info:PChar);

implementation

uses ID3v2, ID3v1, APE;

function GetGenreID(Const Genre:string):byte;
var
  x:integer;
begin
  for x := 0 to MAX_MUSIC_GENRES - 1 do
    if UpperCase(Genre) = UpperCase(MusicGenre[x]) then
    begin
      Result := x;
      exit;
    end;
  Result := 255;
end;


function ExtWODot(Const Ext:string):string;
//  "."   (.mp3 => mp3, .ogg => ogg)
begin
  if Ext = '' then exit;
  if Ext[1] = '.' then result := Copy(Ext,2,length(Ext))
                  else result := Ext;
end;

function ExtToIdx(Const Ext:string):integer;
//    (    )
var
  x: integer;
  nExt: string;
begin
  nExt := AnsiUpperCase(ExtWODot(Ext));
  for x := 0 to FormatList.Count - 1 do
  if nExt = AnsiUpperCase(FormatList.Items[x]) then
  begin
    result := x;
    exit;
  end;
  result := -1;
end;



//////////////////////////////////////////////////////////////////////////
//=====================    ===========================//
//////////////////////////////////////////////////////////////////////////

procedure GetID3v1TagInfo(Const Tag: PID3v1);
//    D3v1
begin
  with Tag^ do
  begin
    GlobalInfBuf.Title := Title;
    GlobalInfBuf.Artist := Artist;
    GlobalInfBuf.Album := Album;
    GlobalInfBuf.Comment := Comment;
    GlobalInfBuf.Genre := Genre;
    GlobalInfBuf.Date := Year;
    GlobalInfBuf.TrackNo := Track;
  end;
end;

procedure GetID3v2TagInfo(Const Tag: PID3v2);
//    D3v1
begin
  with Tag^ do
  begin
    GlobalInfBuf.Title := Title;
    GlobalInfBuf.Artist := Artist;
    GlobalInfBuf.Album := Album;
    GlobalInfBuf.Comment := Comment;
    GlobalInfBuf.Genre := Genre;
    GlobalInfBuf.Date := Year;
    GlobalInfBuf.TrackNo := Track;
  end
end;

function Read_aacTag(Const Info: PBasicInfoRec):boolean; {idx = 0}
//   *.aac 
var
  AAC: PAAC;
begin
  AAC := NewAAC;
  try
    result := AAC.ReadFromFile(Info.FilePath);
    if result then
    begin
      GlobalInfBuf.Time := Round(AAC.Duration);
      GlobalInfBuf.BitRate := AAC.BitRate;

      AAC.ID3v1.ReadFromFile(Info.FilePath);

      if AAC.ID3v2.Exists then GetID3v2TagInfo(AAC.ID3v2)
      else
      if AAC.ID3v1.Exists then GetID3v1TagInfo(AAC.ID3v1);
      
      GlobalInfBuf.TagPr := AAC.ID3v2.Exists or AAC.ID3v1.Exists;
    end;
  finally
    AAC.Free;
  end;
end;

function Read_MpegPlusTag(Const Info: PBasicInfoRec):boolean; {idx = 6}
//   *.mp+ 
//   = APETag -> ID3v2 -> ID3v1
var
  MpegPlus: PMpegPlus;
begin
  MpegPlus := NewMpegPlus;
  try
    result := MpegPlus.ReadFromFile(Info.FilePath);
    if result then
    begin
      GlobalInfBuf.Time := Round(MpegPlus.Duration);
      GlobalInfBuf.BitRate := MpegPlus.BitRate;

      if MpegPlus.APEtag.Exists then
        with MpegPlus.APEtag^ do
        begin
          GlobalInfBuf.Title := Title;
          GlobalInfBuf.Artist := Artist;
          GlobalInfBuf.Album := Album;
          GlobalInfBuf.Comment := Comment;
          GlobalInfBuf.Genre := Genre;
          GlobalInfBuf.Date := Year;
          GlobalInfBuf.TrackNo := Track;
        end
      else if MpegPlus.ID3v2.Exists then GetID3v2TagInfo(MpegPlus.ID3v2)
      else if MpegPlus.ID3v1.Exists then GetID3v1TagInfo(MpegPlus.ID3v1);

      GlobalInfBuf.TagPr := MpegPlus.APEtag.Exists or MpegPlus.ID3v2.Exists or MpegPlus.ID3v1.Exists;
    end;
  finally
    MpegPlus.Free;
  end;
end;

function Read_apeTag(Const Info: PBasicInfoRec):boolean; {idx = 1..2}
//   *.ape  *.mpc 
//   = APETag -> ID3v2 -> ID3v1
var
  Monkey: PMonkey;
begin
  Monkey := NewMonkey;
  try
    result := Monkey.ReadFromFile(Info.FilePath);
    if result then
    begin
      GlobalInfBuf.Time := Round(Monkey.Duration);
      GlobalInfBuf.BitRate := 0;//Monkey.Samples;

      if Monkey.APEtag.Exists then
        with Monkey.APEtag^ do
        begin
          GlobalInfBuf.Title := Title;
          GlobalInfBuf.Artist := Artist;
          GlobalInfBuf.Album := Album;
          GlobalInfBuf.Comment := Comment;
          GlobalInfBuf.Genre := Genre;
          GlobalInfBuf.Date := Year;
          GlobalInfBuf.TrackNo := Track;
        end
      else if Monkey.ID3v2.Exists then GetID3v2TagInfo(Monkey.ID3v2)
      else if Monkey.ID3v1.Exists then GetID3v1TagInfo(Monkey.ID3v1);

      GlobalInfBuf.TagPr := Monkey.APEtag.Exists or Monkey.ID3v2.Exists or Monkey.ID3v1.Exists;
    end
    else
      if AnsiUpperCase(ExtractFileExt(Info.FilePath)) = '.MPC' then result := Read_MpegPlusTag(Info);
  finally
    Monkey.Free;
  end;
end;


function Read_FLACTag(Const Info: PBasicInfoRec):boolean; {idx = 3}
//   *.flac 
var
  FLAC: PFLAC;
begin
  FLAC := NewFLAC;
  try
    result := FLAC.ReadFromFile(Info.FilePath);
    if result then
    begin
      GlobalInfBuf.Time := Round(FLAC.Duration);
      GlobalInfBuf.BitRate := 0;//FLAC.BitRate;

      if FLAC.ID3v2.Exists then       GetID3v2TagInfo(FLAC.ID3v2)
      else if FLAC.ID3v1.Exists then  GetID3v1TagInfo(FLAC.ID3v1);

      GlobalInfBuf.TagPr := FLAC.ID3v2.Exists or FLAC.ID3v1.Exists;
    end;
  finally
    FLAC.Free;
  end;
end;

function Read_MpegTag(Const Info: PBasicInfoRec):boolean; {idx = 4..5}
//   *.mp3  *.mp2 
var
  Mpeg: PMPEGaudio;
begin
  Mpeg := NewMPEGaudio;
  try
    result := Mpeg.ReadFromFile(Info.FilePath);
    if result then
    begin
      Mpeg.ID3v1.ReadFromFile(Info.FilePath);
      GlobalInfBuf.Time := Round(Mpeg.Duration);
      GlobalInfBuf.BitRate := Mpeg.BitRate;

      if Mpeg.ID3v2.Exists then       GetID3v2TagInfo(Mpeg.ID3v2)
      else if Mpeg.ID3v1.Exists then  GetID3v1TagInfo(Mpeg.ID3v1);

      GlobalInfBuf.TagPr := Mpeg.ID3v2.Exists or Mpeg.ID3v1.Exists;
    end;
  finally
    Mpeg.Free;
  end;
end;

function Read_TwinVQTag(Const Info: PBasicInfoRec):boolean; {idx = 7}
//   *.vqf 
var
  TwinVQ: PTwinVQ;
begin
  TwinVQ := NewTwinVQ;
  try
    result := TwinVQ.ReadFromFile(Info.FilePath);
    if result then
    begin
      GlobalInfBuf.Time := Round(TwinVQ.Duration);
      GlobalInfBuf.BitRate := TwinVQ.BitRate;

      if TwinVQ.Exists then
        with TwinVQ^ do
        begin
          GlobalInfBuf.Title := Title;
          GlobalInfBuf.Artist := Author;
          GlobalInfBuf.Album := Album;
          GlobalInfBuf.Comment := Comment;
          GlobalInfBuf.Genre := '';
          GlobalInfBuf.Date := '';
          GlobalInfBuf.TrackNo := 255;  // 
        end;

      GlobalInfBuf.TagPr := TwinVQ.Exists;
    end;
  finally
    TwinVQ.Free;
  end;
end;

function Read_wmaTag(Const Info: PBasicInfoRec):boolean; {idx = 8..9}
//   *.wma 
var
  wma: Pwma;
begin
  wma := NewWMA;
  //wma.Create;
  try
    result := wma.LoadFromFile(Info.FilePath);
    if result then
    begin
      GlobalInfBuf.Time := Round(wma.Duration);
      GlobalInfBuf.BitRate := wma.BitRate;
      with wma^ do
      begin
        GlobalInfBuf.Title := Title;
        if Title = 'MCDI' then exit; //  .
        GlobalInfBuf.Artist := Artist;
        GlobalInfBuf.Album := Album;
        GlobalInfBuf.Comment := Comment;
        GlobalInfBuf.Genre := Genre;
        GlobalInfBuf.Date := Year;
        GlobalInfBuf.TrackNo := Track;
      end;
      GlobalInfBuf.TagPr := true;
    end;
  finally
    wma.Free;
  end;
end;

function Read_CDATag(Const Info: PBasicInfoRec):boolean; {idx = 10}
//   *.CDAtrack 
var
  CDAtrack: PCDAtrack;
begin
  CDAtrack := NewCDAtrack;
  //CDAtrack.Create;
  try
    result := CDAtrack.ReadFromFile(Info.FilePath);
    if result then
    begin
      GlobalInfBuf.Time := Round(CDAtrack.Duration);
      with CDAtrack^ do
      begin
        GlobalInfBuf.Title := Title;
        GlobalInfBuf.Artist := Artist;
        GlobalInfBuf.Album := Album;
        GlobalInfBuf.TrackNo := Track;
      end;
      GlobalInfBuf.TagPr := true;
    end;
  finally
    CDAtrack.Free;
  end;
end;

//////////////////////////////////////////////////////////////////////////
//===end===============    ===========================//
//////////////////////////////////////////////////////////////////////////




//////////////////////////////////////////////////////////////////////////
//=====================    ===========================//
//////////////////////////////////////////////////////////////////////////

function CanUseWID3v1(Const Info: PBasicInfoRec):boolean;
//      Info  ID3v1
begin
  result := ((length(Info.Title) <= 30) and
             (length(Info.Artist) <= 30) and
             (length(Info.Album) <= 30) and
             (length(Info.Comment) <= 30) and
             ((Trim(Info.Genre) <> '') and (GetGenreID(Info.Genre) <> 255)));
end;

function WriteID3v1(Const Tag: PID3v1; Const Info: PBasicInfoRec):boolean;
begin
  with Tag^ do
  begin
    Title := Info.Title;
    Artist := Info.Artist;
    Album := Info.Album;
    Comment := Info.Comment;
    GenreID := GetGenreID(Info.Genre);
    Year := Info.Date;
    Track := Info.TrackNo;
    Result := SaveToFile(Info.FilePath) and CanUseWID3v1(Info);
    //      ID3v1,    ID3v1
    //    result = false -    ID3v2
  end;
end;

function WriteID3v2(Const Tag: PID3v2; Const Info: PBasicInfoRec):boolean;
begin
  with Tag^ do
  begin
    Title := Info.Title;
    Artist := Info.Artist;
    Album := Info.Album;
    Comment := Info.Comment;
    Genre := Info.Genre;
    Year := Info.Date;
    Track := Info.TrackNo;
    Result := SaveToFile(Info.FilePath);
  end;
end;

function Write_aacTag(Const Info: PBasicInfoRec):boolean; {idx = 0}
//   *.aac 
var
  AAC: PAAC;
begin
  AAC := NewAAC;
  Result := false;
  try
    AAC.ReadFromFile(Info.FilePath);
    AAC.ID3v1.ReadFromFile(Info.FilePath);

    if AAC.ID3v1.Exists then result := WriteID3v1(AAC.ID3v1, Info);
    if (AAC.ID3v2.Exists) or (not Result) then result := WriteID3v2(AAC.ID3v2, Info);
    //       ID3v1  ID3v2
  finally
    AAC.Free;
  end;
end;

function Write_MpegPlusTag(Const Info: PBasicInfoRec):boolean; {idx = 6}
//   *.mp+ 
//   = APETag -> ID3v2 -> ID3v1
var
  MpegPlus: PMpegPlus;
begin
  MpegPlus := NewMpegPlus;
  Result := false;
  try
    MpegPlus.ReadFromFile(Info.FilePath);


    if MpegPlus.ID3v1.Exists then result := WriteID3v1(MpegPlus.ID3v1, Info);
    if MpegPlus.ID3v2.Exists then result := WriteID3v2(MpegPlus.ID3v2, Info);
    if (MpegPlus.APEtag.Exists) or (not Result) then
      with MpegPlus.APEtag^ do
      begin
        Title := Info.Title;
        Artist := Info.Artist;
        Album := Info.Album;
        Comment := Info.Comment;
        Genre := Info.Genre;
        Year := Info.Date;
        Track := Info.TrackNo;
        Result := SaveToFile(Info.FilePath);
      end;

  finally
    MpegPlus.Free;
  end;
end;

function Write_apeTag(Const Info: PBasicInfoRec):boolean; {idx = 1..2}
//   *.ape  *.mpc 
//   = APETag -> ID3v2 -> ID3v1
var
  Monkey: PMonkey;
begin
  Monkey := NewMonkey;
  try
    Result := Monkey.ReadFromFile(Info.FilePath);
    if result then
    begin

      if Monkey.ID3v1.Exists then result := WriteID3v1(Monkey.ID3v1, Info);
      if Monkey.ID3v2.Exists then result := WriteID3v2(Monkey.ID3v2, Info);

      if Monkey.APEtag.Exists then
        with Monkey.APEtag^ do
        begin
          Title := Info.Title;
          Artist := Info.Artist;
          Album := Info.Album;
          Comment := Info.Comment;
          Genre := Info.Genre;
          Year := Info.Date;
          Track := Info.TrackNo;
          Result := SaveToFile(Info.FilePath);
        end;

    end
    else
      if AnsiUpperCase(ExtractFileExt(Info.FilePath)) = '.MPC' then result := Write_MpegPlusTag(Info);
  finally
    Monkey.Free;
  end;
end;


function Write_FLACTag(Const Info: PBasicInfoRec):boolean; {idx = 3}
//   *.flac 
var
  FLAC: PFLAC;
begin
  FLAC := NewFLAC;
  Result := false;
  try
    FLAC.ReadFromFile(Info.FilePath);

    if FLAC.ID3v1.Exists then result := WriteID3v1(FLAC.ID3v1, Info);
    if (FLAC.ID3v2.Exists) or (not Result) then result := WriteID3v2(FLAC.ID3v2, Info);

  finally
    FLAC.Free;
  end;
end;

function Write_MpegTag(Const Info: PBasicInfoRec):boolean; {idx = 4..5}
//   *.mp3  *.mp2 
var
  Mpeg: PMPEGaudio;
begin
  Mpeg := NewMPEGaudio;
  Result := false;
  try
    Mpeg.ReadFromFile(Info.FilePath);

    if Mpeg.ID3v1.Exists then result := WriteID3v1(Mpeg.ID3v1, Info);
    if (Mpeg.ID3v2.Exists) or (not Result) then result := WriteID3v2(Mpeg.ID3v2, Info);

  finally
    Mpeg.Free;
  end;
end;

function Write_TwinVQTag(Const Info: PBasicInfoRec):boolean; {idx = 7}
//   *.vqf 
var
  TwinVQ: PTwinVQ;
begin
  TwinVQ := NewTwinVQ;
  try
    TwinVQ.ReadFromFile(Info.FilePath);
    with TwinVQ^ do
    begin
      Title := Info.Title;
      Author := Info.Artist;
      Album := Info.Album;
      Comment := Info.Comment;
      Result := SaveToFile(Info.FilePath);
    end;

  finally
    TwinVQ.Free;
  end;
end;

function Write_wmaTag(Const Info: PBasicInfoRec):boolean; {idx = 8..9}
//   *.wma 
var
  wma: Pwma;
begin
  result := false;
  wma := NewWMA;
  try
    wma.LoadFromFile(Info.FilePath);
    with wma^ do
    begin
      if Title = 'MCDI' then exit; //  .
      Title := Info.Title;
      Artist := Info.Artist;
      Album := Info.Album;
      Comment := Info.Comment;
      Genre := Info.Genre;
      Year := Info.Date;
      Track := Info.TrackNo;
      Result := SaveToFile(Info.FilePath);
    end;

  finally
    wma.Free;
  end;
end;

//////////////////////////////////////////////////////////////////////////
//===end===============    ===========================//
//////////////////////////////////////////////////////////////////////////





//////////////////////////////////////////////////////////////////////////
//============     =====================//
//////////////////////////////////////////////////////////////////////////

function CheckedString(Const FieldName, s:string; Const NeedLineBreak:boolean = true):string;
begin
 Result := s;
 if Result <> '' then
 begin
   Result := FieldName + FeeldNameToFieldSeparator + Result;
   if NeedLineBreak then Result := Result + LineSeparator;
 end;
end;

function GetID3v2Info(Const ID3v2: PID3v2):string;
begin
  Result := '';
  with ID3v2^ do
  begin
    if not Exists then exit;
    Result := Result + CheckedString(fn_Song_title, Title);
    Result := Result + CheckedString(fn_Artist_name, Artist);
    Result := Result + CheckedString(fn_Album_title, Album);
    Result := Result + CheckedString(fn_Track_number, TrackString);
    Result := Result + CheckedString(fn_Release_year, Year);
    Result := Result + CheckedString(fn_Genre_name, Genre);
    Result := Result + CheckedString(fn_Comment, Comment);
    Result := Result + CheckedString(fn_Composer, Composer);
    Result := Result + CheckedString(fn_Encoder, Encoder);
    Result := Result + CheckedString(fn_Copyright, Copyright);
    Result := Result + CheckedString(fn_Language, Language);
    Result := Result + CheckedString(fn_Language, Link, false);
  end;
end;

function GetID3v1Info(Const ID3v1: PID3v1):string;
begin
  Result := '';
  with ID3v1^ do
  begin
    if not Exists then exit;
    Result := Result + CheckedString(fn_Song_title, Title);
    Result := Result + CheckedString(fn_Artist_name, Artist);
    Result := Result + CheckedString(fn_Album_title, Album);
    Result := Result + CheckedString(fn_Track_number, Int2Str(Track));
    Result := Result + CheckedString(fn_Release_year, Year);
    Result := Result + CheckedString(fn_Genre_name, Genre);
    Result := Result + CheckedString(fn_Comment, Comment, false);
  end;
end;

function GetAPETagInfo(Const APE: PAPE):string;
begin
  Result := '';
  with APE^ do
  begin
    if not Exists then exit;
    Result := Result + CheckedString(fn_Song_title, Title);
    Result := Result + CheckedString(fn_Artist_name, Artist);
    Result := Result + CheckedString(fn_Album_title, Album);
    Result := Result + CheckedString(fn_Track_number, Int2Str(Track));
    Result := Result + CheckedString(fn_Release_year, Year);
    Result := Result + CheckedString(fn_Genre_name, Genre);
    Result := Result + CheckedString(fn_Copyright, Copyright);
    Result := Result + CheckedString(fn_Comment, Comment, false);
  end;
end;

function GetTwinVQTagInfo(Const TwinVQ: PTwinVQ):string;
begin
  Result := '';
  with TwinVQ^ do
  begin
    if not Exists then exit;
    Result := Result + CheckedString(fn_Song_title, Title);
    Result := Result + CheckedString(fn_Artist_name, Author);
    Result := Result + CheckedString(fn_Album_title, Album);
    Result := Result + CheckedString(fn_Copyright, Copyright);
    Result := Result + CheckedString(fn_Comment, Comment, false);
  end;
end;

function GetWMATagInfo(Const WMA: PWMA):string;
begin
  Result := '';
  with WMA^ do
  begin
    Result := Result + CheckedString(fn_Song_title, Title);
    Result := Result + CheckedString(fn_Artist_name, Artist);
    Result := Result + CheckedString(fn_Track_number, Int2Str(Track));
    Result := Result + CheckedString(fn_Album_title, Album);
    Result := Result + CheckedString(fn_Copyright, Copyright);
    Result := Result + CheckedString(fn_Release_year, Year);
    Result := Result + CheckedString(fn_Composer, Composer);
    Result := Result + CheckedString(fn_Rating, Rating);
    Result := Result + CheckedString(fn_Copyright, Copyright);
    Result := Result + CheckedString(fn_Comment, Comment);
    Result := Result + CheckedString(fn_Lyrics, Lyrics, false);
  end;
end;

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

procedure Info_aac(Const FileName:PChar;Var info:PChar);
var
  AAC: PAAC;
begin
  AAC := NewAAC;
  FileInfoBuf := '';
  try
    if AAC.ReadFromFile(FileName) then
    with AAC^ do
    begin
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Header_type_name, HeaderType);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_MPEG_version_name, MPEGVersion);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Profile_name, Profile);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Bit_rate, BitRateType);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Sample_rate, Int2Str(SampleRate));
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Bit_rate, Int2Str(BitRate));
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Channels, Int2Str(Channels));
      FileInfoBuf := FileInfoBuf + LineSeparator;
      FileInfoBuf := FileInfoBuf + CheckedString('ID3v2',GetID3v2Info(ID3v2));
      FileInfoBuf := FileInfoBuf + CheckedString('ID3v1',GetID3v1Info(ID3v1));
    end;
  finally
    AAC.Free;
  end;
end;

procedure Info_MpegPlus(Const FileName:PChar;Var info:PChar);
var
  MpegPlus: PMpegPlus;
begin
  MpegPlus := NewMpegPlus;
  FileInfoBuf := '';
  try
    if MpegPlus.ReadFromFile(FileName) then
    with MpegPlus^ do
    begin
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Profile_name, Profile);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Channel_mode, ChannelMode);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Bit_rate, Int2Str(BitRate));
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Encoder_version, Int2Str(StreamVersion));
      FileInfoBuf := FileInfoBuf + LineSeparator;
      FileInfoBuf := FileInfoBuf + CheckedString('ID3v2',GetID3v2Info(ID3v2));
      FileInfoBuf := FileInfoBuf + CheckedString('ID3v1',GetID3v1Info(ID3v1));
      FileInfoBuf := FileInfoBuf + CheckedString('APE tag',GetAPETagInfo(APEtag));
    end else Info_ape(FileName,info);
  finally
    MpegPlus.Free;
  end;
end;

procedure Info_ape(Const FileName:PChar;Var info:PChar);
var
  Monkey: PMonkey;
begin
  Monkey := NewMonkey;
  FileInfoBuf := '';
  try
    if Monkey.ReadFromFile(FileName) then
    with Monkey^ do
    begin
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Compression_level, Compression);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Channel_mode, ChannelMode);
      FileInfoBuf := FileInfoBuf + LineSeparator;
      FileInfoBuf := FileInfoBuf + CheckedString('ID3v2',GetID3v2Info(ID3v2));
      FileInfoBuf := FileInfoBuf + CheckedString('ID3v1',GetID3v1Info(ID3v1));
      FileInfoBuf := FileInfoBuf + CheckedString('APE tag',GetAPETagInfo(APEtag));
    end;
  finally
    Monkey.Free;
  end;
end;

procedure Info_FLAC(Const FileName:PChar;Var info:PChar);
var
  FLAC: PFLAC;
begin
  FLAC := NewFLAC;
  FileInfoBuf := '';
  try
    if FLAC.ReadFromFile(FileName) then
    with FLAC^ do
    begin
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Song_title, Title);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Artist_name, Artist);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Track_number, Track);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Album_title, Album);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Genre_name, Genre);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Release_year, Year);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Copyright, Copyright);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Comment, Comment);
    end;
  finally                          
    FLAC.Free;                     
  end;
end;

procedure Info_MpegAudio(Const FileName:PChar;Var info:PChar);
var
  MpegAudio: PMpegAudio;
begin
  MpegAudio := NewMpegAudio;
  FileInfoBuf := '';
  try
    if MpegAudio.ReadFromFile(FileName) then
    with MpegAudio^ do
    begin
      FileInfoBuf := FileInfoBuf + CheckedString(fn_MPEG_version_name, Version);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_MPEG_layer_name, Layer);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Channel_mode, ChannelMode);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Encoder, Encoder);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Sample_rate, Int2Str(SampleRate));
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Bit_rate, Int2Str(BitRate), false);
      if vbr.Found then FileInfoBuf := FileInfoBuf + ' (VBR)' + LineSeparator
                   else FileInfoBuf := FileInfoBuf + LineSeparator;
      FileInfoBuf := FileInfoBuf + LineSeparator;
      FileInfoBuf := FileInfoBuf + CheckedString('ID3v2',GetID3v2Info(ID3v2));
      FileInfoBuf := FileInfoBuf + CheckedString('ID3v1',GetID3v1Info(ID3v1));
    end;
  finally
    MpegAudio.Free;
  end;
end;

procedure Info_TwinVQ(Const FileName:PChar;Var info:PChar);
var
  TwinVQ: PTwinVQ;
begin
  TwinVQ := NewTwinVQ;
  FileInfoBuf := '';
  try
    if TwinVQ.ReadFromFile(FileName) then
    with TwinVQ^ do
    begin
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Channel_mode, ChannelMode);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Sample_rate, Int2Str(SampleRate));
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Bit_rate, Int2Str(BitRate));
      FileInfoBuf := FileInfoBuf + LineSeparator;
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Song_title, Title);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Artist_name, Author);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Album_title, Album);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Copyright, Copyright);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Comment, Comment);
    end;
  finally
    TwinVQ.Free;
  end;
end;

procedure Info_WMA(Const FileName:PChar;Var info:PChar);
var
  WMA: PWMA;
begin
  WMA := NewWMA;
  //WMA.Create;
  FileInfoBuf := '';
  try
    if WMA.LoadFromFile(FileName) then
    with WMA^ do
    begin
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Encoder_version, Version);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Channel_mode, ChannelsMode);
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Sample_rate, Int2Str(SampleRate));
      FileInfoBuf := FileInfoBuf + CheckedString(fn_Bit_rate, Int2Str(BitRate));
      FileInfoBuf := FileInfoBuf + LineSeparator;
      FileInfoBuf := FileInfoBuf + CheckedString('WMA tag', GetWMATagInfo(WMA));
    end;
  finally
    WMA.Free;
  end;
end;


//////////////////////////////////////////////////////////////////////////
//===end======     =====================//
//////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////
//=====================   ========================//
//////////////////////////////////////////////////////////////////////////

procedure SetDllExt;
var
  x:integer;
begin
  dllExt := FormatList.Items[0];
  for x := 1 to FormatList.Count - 1 do dllExt := dllExt + ','  + FormatList.Items[x];
end;

procedure FillFormats;
//    
begin
  with FormatList^ do
  begin
    Add('aac');  // 0
    Add('ape');  // 1
    Add('mpc');  // 2
    Add('flac'); // 3
    Add('mp3');  // 4
    Add('mp2');  // 5
    Add('mp+');  // 6
    Add('vqf');  // 7
    Add('wma');  // 8
    Add('asf');  // 9
    Add('cda');  // 10
    SetDllExt;
  end;
end;

initialization
  FormatList := NewStrList;
  FillFormats;

finalization
  if Assigned(FormatList) then Free_And_Nil(FormatList);

//=====================================================================//

end.
